from django.conf.urls import url

from rest_framework import routers

from bulbs.super_features.views import (RelationViewSet,
                                        RelationOrderingViewSet,
                                        SetChildrenDatesViewSet)
from bulbs.super_features.viewsets import SuperFeatureViewSet

api_v1_router = routers.DefaultRouter()
api_v1_router.register(
    r"super-feature",
    SuperFeatureViewSet,
    base_name="super-feature"
)

urlpatterns = [
    url(
        r'super-feature/(?P<pk>\d+)/relations/?$',
        RelationViewSet.as_view(),
        name='super-feature-relations'
    ),
    url(
        r'super-feature/(?P<pk>\d+)/relations/ordering/?$',
        RelationOrderingViewSet.as_view(),
        name='super-feature-relations-ordering'
    ),
    url(
        r'super-feature/(?P<pk>\d+)/set-children-dates',
        SetChildrenDatesViewSet.as_view(),
        name='super-feature-set-children-dates'
    )
]

urlpatterns += api_v1_router.urls



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import json_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('special_coverage', '0011_remove_specialcoverage_campaign'),
    ]

    operations = [
        migrations.AddField(
            model_name='specialcoverage',
            name='super_features',
            field=json_field.fields.JSONField(default=[], help_text='Enter a valid JSON object', blank=True),
        ),
    ]



from django.utils import timezone

from elasticsearch_dsl.filter import Range


def Closed():  # noqa
    end_date_params = {
        "lte": timezone.now(),
    }

    return Range(end_date=end_date_params)



from rest_framework import filters, viewsets
from rest_framework.permissions import IsAdminUser

from bulbs.api.permissions import CanEditContent
from bulbs.super_features.utils import get_superfeature_model, get_superfeature_serializer
from bulbs.super_features.filters import SuperFeatureFilter


SUPERFEATURE_MODEL = get_superfeature_model()
SUPERFEATURE_SERIALIZER = get_superfeature_serializer()


class SuperFeatureViewSet(viewsets.ModelViewSet):

    model = SUPERFEATURE_MODEL
    queryset = SUPERFEATURE_MODEL.objects.filter(parent__isnull=True)
    serializer_class = SUPERFEATURE_SERIALIZER
    permission_classes = [IsAdminUser, CanEditContent]
    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter,
        filters.DjangoFilterBackend,
    )
    filter_class = SuperFeatureFilter
    search_fields = ("title",)
    ordering_fields = ("title",)
    paginate_by = 20



from rest_framework import viewsets

from bulbs.api.permissions import CanEditCmsNotifications
from bulbs.cms_notifications.models import CmsNotification
from bulbs.cms_notifications.serializers import CmsNotificationSerializer


class CmsNotificationsViewSet(viewsets.ModelViewSet):
    """
    viewset for `bulbs.cms_notifications.CmsNotification` model
    """

    queryset = CmsNotification.objects.all()
    serializer_class = CmsNotificationSerializer
    permission_classes = [CanEditCmsNotifications]


# defines a generic django view mapping for the CMS Notifications
cms_notifications_view = CmsNotificationsViewSet.as_view({
    'get': 'list',
    'post': 'create',
    'put': 'update',
    'delete': 'destroy'
})



from datetime import date, datetime
from dateutil import tz
from importlib import import_module
from six import string_types, text_type, binary_type
from six.moves.urllib.parse import urlencode

from django import template
from django.conf import settings
from django.http import Http404, HttpResponseRedirect
from django.utils import timezone
from django.apps import apps
from django.shortcuts import get_object_or_404


INVALID_TEMPLATE_CHOICE = """Choices for template values must be non-zero integers."""


def get_central_now():
    utc_now = timezone.now()
    central_tz = tz.gettz('America/Chicago')
    central_now = utc_now.astimezone(central_tz)
    return central_now


def today():
    # catching this because settings.TODAY might literally be None in tests
    # getattr will return None if the value is set to None
    return getattr(settings, "TODAY", get_central_now())


def today_as_utc_datetime():
    """Datetime/Date comparisons aren't great, and someone might configure TODAY, to be a date."""
    now = today()
    if not isinstance(now, datetime) and isinstance(now, date):
        now = datetime.combine(now, datetime.min.time())
        now = now.replace(tzinfo=tz.gettz('UTC'))
    return now


def get_query_params(request):
    try:
        return request.query_params
    except:
        return request.QUERY_PARAMS


def get_request_data(request):
    try:
        return request.data
    except:
        return request.DATA


def is_str(value):
    return isinstance(value, (string_types, text_type, binary_type))


def is_valid_digit(value):
    if isinstance(value, (int, float)):
        return True
    elif is_str(value):
        return value.isdigit()
    return False


def datetime_to_epoch_seconds(value):
    epoch = datetime.utcfromtimestamp(0).replace(tzinfo=timezone.utc)
    return (value - epoch).total_seconds()


def get_template_choices():
    configured_templates = getattr(settings, "BULBS_TEMPLATE_CHOICES", ())
    for choice in configured_templates:
        if choice[0] == 0:
            raise ValueError(INVALID_TEMPLATE_CHOICE.format(choice))
        elif type(choice[0]) != int:
            raise ValueError(INVALID_TEMPLATE_CHOICE.format(choice))
    return ((0, None),) + configured_templates


def get_overridable_template_name(parent_name, child_name):
    get_template = template.loader.get_template

    try:
        child_template = get_template(child_name)
    except template.loader.TemplateDoesNotExist:
        child_template = None

    if child_template:
        bad_inheritance_message = "{} MUST extend {}".format(child_name, parent_name)

        extends_node = child_template.template.nodelist.get_nodes_by_type(
            template.loader_tags.ExtendsNode)

        if not extends_node[0]:
            raise Exception(bad_inheritance_message)

        if str(extends_node[0].parent_name) != "'{}'".format(parent_name):
            raise Exception(bad_inheritance_message)

    if child_template:
        template_name = child_name
    else:
        template_name = parent_name

    return template_name


def get_video_object_from_videohub_id(videohub_id):
    video_model = apps.get_model(settings.VIDEO_MODEL)
    return get_object_or_404(video_model, videohub_ref_id=int(videohub_id))


def redirect_unpublished_to_login_or_404(request, next_url, next_params=None):
    redirect_unpublished = getattr(settings, "REDIRECT_UNPUBLISHED_TO_LOGIN", True)
    if not request.user.is_authenticated() and redirect_unpublished:
        if next_params:
            next_url += '?' + urlencode(next_params)
        return HttpResponseRedirect("{}?{}".format(settings.LOGIN_URL,
                                                   urlencode({'next': next_url})))
    else:
        raise Http404


def import_class(name):
    """Load class from fully-qualified python module name.

    ex: import_class('bulbs.content.models.Content')
    """

    module, _, klass = name.rpartition('.')
    mod = import_module(module)
    return getattr(mod, klass)



from django import template

register = template.Library()


@register.inclusion_tag('videos/series-video-list.html', takes_context=True)
def series_video_list(context, channel_slug):
    context["channel_slug"] = channel_slug
    return context


@register.inclusion_tag('videos/series-page.html', takes_context=True)
def series_video_page(context, series_slug):
    context["series_slug"] = series_slug
    return context


@register.inclusion_tag('videos/popular-series.html', takes_context=True)
def popular_series_list(context, channel_slug):
    context["channel_slug"] = channel_slug
    return context


@register.inclusion_tag('videos/video-detail.html', takes_context=True)
def video_detail_page(context, series_slug):
    context["series_slug"] = series_slug
    return context



from django.conf import settings
from django.db import models

from bulbs.content.models import Content
from bulbs.liveblog.utils import get_liveblog_author_model


class AbstractLiveBlog(models.Model):
    """Base mixin inherited by each property's concrete LiveBlog implementation (along with base
    content class).
    """

    pinned_content = models.ForeignKey(Content, blank=True, null=True,
                                       related_name='liveblog_pinned')
    recirc_content = models.ManyToManyField(Content, related_name='liveblog_recirc')

    class Meta:
        abstract = True


class LiveBlogEntry(models.Model):

    liveblog = models.ForeignKey(settings.BULBS_LIVEBLOG_MODEL, related_name='entries')

    published = models.DateTimeField(blank=True, null=True)
    authors = models.ManyToManyField(get_liveblog_author_model())
    headline = models.CharField(max_length=255)
    body = models.TextField(blank=True)

    recirc_content = models.ManyToManyField(Content, related_name='liveblog_entry_recirc')


class LiveBlogResponse(models.Model):

    entry = models.ForeignKey(LiveBlogEntry, related_name='responses')
    ordering = models.IntegerField(blank=True, null=True, default=None)

    internal_name = models.CharField(max_length=255, blank=True, null=True)
    author = models.ForeignKey(get_liveblog_author_model())
    body = models.TextField(blank=True)

    class Meta:
        ordering = ['ordering']



from rest_framework import serializers

from bulbs.content.models import Content

from .models import LiveBlogEntry, LiveBlogResponse


class LiveBlogResponseSerializer(serializers.ModelSerializer):

    class Meta:
        model = LiveBlogResponse
        fields = ('author', 'body', 'internal_name')


class LiveBlogEntrySerializer(serializers.ModelSerializer):

    responses = LiveBlogResponseSerializer(many=True)

    recirc_content = serializers.PrimaryKeyRelatedField(many=True, queryset=Content.objects.all())

    class Meta:
        model = LiveBlogEntry

    def create(self, validated_data):
        responses = validated_data.pop('responses')

        entry = super(LiveBlogEntrySerializer, self).create(validated_data)

        # Create nested responses
        for idx, response in enumerate(responses):
            LiveBlogResponse.objects.create(entry=entry, ordering=idx, **response)

        return entry

    def update(self, instance, validated_data):
        responses = validated_data.pop('responses')

        entry = super(LiveBlogEntrySerializer, self).update(instance, validated_data)

        # Swap in brand new list of responses (easier than trying to match up)
        entry.responses.all().delete()
        for idx, response in enumerate(responses):
            LiveBlogResponse.objects.create(entry=entry, ordering=idx, **response)

        return entry



from rest_framework import filters, viewsets
from rest_framework_extensions.cache.mixins import CacheResponseMixin
from rest_framework.permissions import AllowAny, IsAdminUser

from bulbs.api.permissions import CanEditContent
from .filters import IfModifiedSinceFilterBackend, LiveBlogFilterBackend
from .models import LiveBlogEntry
from .serializers import LiveBlogEntrySerializer


class LiveBlogEntryViewSet(viewsets.ModelViewSet):

    queryset = LiveBlogEntry.objects.all()
    # model = LiveBlogEntry
    serializer_class = LiveBlogEntrySerializer
    permission_classes = [IsAdminUser, CanEditContent]
    filter_backends = (
        filters.OrderingFilter,
        LiveBlogFilterBackend,
    )
    ordering_fields = (
        "published",
    )

    # Avoid immediate need for FE pagination
    paginate_by = 500


class PublicLiveBlogEntryViewSet(CacheResponseMixin, LiveBlogEntryViewSet):

    permission_classes = (AllowAny,)
    filter_backends = (
        filters.OrderingFilter,
        IfModifiedSinceFilterBackend,
        LiveBlogFilterBackend,
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations

# NOTE: Referring to the Special Coverage model directly in this migration stopped
#       newer migrations from being run. This is fine though, since save() only needed to be run once


class Migration(migrations.Migration):

    dependencies = [
        ('special_coverage', '0008_auto_20160118_1337'),
    ]

    operations = [
    ]



from rest_framework import filters
from rest_framework.exceptions import ParseError

from django.utils import dateparse


logger = __import__('logging').getLogger(__name__)


class IfModifiedSinceFilterBackend(filters.BaseFilterBackend):
    """
    Optionally exclude older requests.

    Expects ISO 8601-formatted "if_modified_since" query param.
    """
    def filter_queryset(self, request, queryset, view):
        if_modified_since = request.GET.get('if_modified_since')
        if if_modified_since:
            when = dateparse.parse_datetime(if_modified_since)
            if when:
                return queryset.filter(published__gt=when)
            else:
                raise ParseError('Bad "if_modified_since" date: {}'.format(if_modified_since))
        else:
            return queryset


class LiveBlogFilterBackend(filters.BaseFilterBackend):
    """
    Optionally include only entries belonging to specified LiveBlog.
    """
    def filter_queryset(self, request, queryset, view):
        liveblog_id = request.GET.get('liveblog_id')
        if liveblog_id:
            queryset = queryset.filter(liveblog_id=liveblog_id)

        return queryset



import json

from django.template import Library, Context
from django.template.loader import get_template


register = Library()


@register.simple_tag(takes_context=True)
def ads_targeting(context):
    return get_template(
        "ads/ads-targeting.html"
    ).render(
        Context({
            "targeting": context.get("targeting")
        })
    )


@register.simple_tag
def dfp_ad(ad_unit, css_class=''):
    template = get_template("ads/dfp.html")
    context = {
        "ad_unit": ad_unit,
        "css_class": css_class
    }

    return template.render(Context(context))


@register.simple_tag(takes_context=True)
def targeting(context):
    targeting_template = "<script>var TARGETING={};</script>"  # Cute, huh?
    if "targeting" not in context:
        return targeting_template
    return targeting_template.format(json.dumps(context["targeting"]))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings

from bulbs.liveblog.utils import get_liveblog_author_model


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(get_liveblog_author_model()),
        migrations.swappable_dependency(settings.BULBS_LIVEBLOG_MODEL),
        ('content', '0012_auto_20160615_1605'),
    ]

    operations = [
        migrations.CreateModel(
            name='LiveBlogEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, verbose_name='ID', serialize=False)),
                ('published', models.DateTimeField(null=True, blank=True)),
                ('headline', models.CharField(max_length=255)),
                ('body', models.TextField(blank=True)),
                ('authors', models.ManyToManyField(to=get_liveblog_author_model())),
                ('liveblog', models.ForeignKey(to=settings.BULBS_LIVEBLOG_MODEL, related_name='entries')),
                ('recirc_content', models.ManyToManyField(to='content.Content', related_name='liveblog_entry_recirc')),
            ],
        ),
        migrations.CreateModel(
            name='LiveBlogResponse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, verbose_name='ID', serialize=False)),
                ('ordering', models.IntegerField(null=True, blank=True, default=None)),
                ('body', models.TextField(blank=True)),
                ('author', models.ForeignKey(to=get_liveblog_author_model())),
                ('entry', models.ForeignKey(to='liveblog.LiveBlogEntry', related_name='responses')),
            ],
        ),
    ]



from rest_framework import serializers

from bulbs.content.serializers import ContentSerializer
from .models import BaseInfographic
from .utils import get_data_serializer


class InfographicDataField(serializers.Field):

    def to_internal_value(self, data):
        serializer = get_data_serializer(int(self.parent.initial_data.get("infographic_type")))
        return serializer().to_internal_value(data)

    def to_representation(self, obj):
        if hasattr(self.parent, 'initial_data'):
            infographic_type = self.parent.initial_data.get('infographic_type', None)
        else:
            infographic_type = getattr(self.parent, 'infographic_type')
        serializer_class = get_data_serializer(int(infographic_type))

        # TODO: figure out a more elegant transition for elasticsearch calls.
        if type(obj).__name__ == 'InnerObjectWrapper':
            obj = obj.__dict__.get('_d_')
        serializer = serializer_class(data=obj)
        serializer.is_valid(raise_exception=True)
        return serializer.data


class InfographicSerializer(ContentSerializer):

    data = InfographicDataField(required=False)

    class Meta:
        model = BaseInfographic

    def to_representation(self, obj):
        self.infographic_type = getattr(obj, 'infographic_type')
        return super(InfographicSerializer, self).to_representation(obj)



from django.db import models


class SpecialCoverageManager(models.Manager):
    def get_by_identifier(self, identifier):
        identifier_id = identifier.split(".")[-1]
        return super(SpecialCoverageManager, self).get_queryset().get(id=identifier_id)



from collections import OrderedDict
import csv

from rest_framework_csv.renderers import CSVRenderer
from rest_framework_csv.misc import Echo
import six

import django.db.models.query

from djes.utils.query import batched_queryset


# DJES LazySearch iteration is a hot mess, and by default will only iterate through a full page (10
# items). This works for now, but eventually we need to just fix DJES
class LazySearchIterator(object):

    def __init__(self, queryset):
        self.queryset = queryset

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        return next(self.queryset)


class CSVStreamingRenderer(CSVRenderer):
    """
        Renders serialized *data* into CSV to be used with Django
        StreamingHttpResponse. We need to return a generator here, so Django
        can iterate over it, rendering and returning each line.

        Based on rest_framework_csv.renderers.CSVStreamingRenderer, but can't use that b/c the
        `tabilize()` call is not iterator-friendly.
    """

    header_fields = {}

    def render(self, data, media_type=None, renderer_context={}):

        queryset = data['queryset']
        serializer = data['serializer']
        context = data['context']

        csv_buffer = Echo()
        csv_writer = csv.writer(csv_buffer)

        # Header row
        if queryset.count():
            yield csv_writer.writerow(self.header_fields.values())

        # Need to efficiently page through querysets
        if isinstance(queryset, django.db.models.query.QuerySet):
            queryset = batched_queryset(queryset, chunksize=25)
        else:
            # This should be build into LazySearch object, but it's not...
            queryset = LazySearchIterator(queryset)

        # Data rows
        for item in queryset:
            items = serializer(item, context=context).data
            # Sort by `header_fields` ordering
            ordered = [items[column] for column in self.header_fields]
            yield csv_writer.writerow([
                elem.encode('utf-8') if isinstance(elem, six.text_type) and six.PY2 else elem
                for elem in ordered
            ])


class ContentReportingRenderer(CSVStreamingRenderer):

    header_fields = OrderedDict([
        ('published', 'Publish Date'),
        ('id', 'Content ID'),
        ('title', 'Title'),
        ('feature_type', 'Feature Type'),
        ('url', 'URL'),
        ('content_type', 'Content Type'),
        ('authors', 'Authors'),
        ('video_id', 'Video ID'),
        ('value', 'Value'),
    ])


class ContributionReportingRenderer(CSVStreamingRenderer):

    header_fields = OrderedDict([
        ('publish_date', 'Publish Date'),
        ('first_name', 'First Name'),
        ('last_name', 'Last Name'),
        ('id', 'Content ID'),
        ('title', 'Title'),
        ('feature_type', 'Feature Type'),
        ('rate', 'Rate'),
        ('payroll_name', 'Payroll Name'),
    ])


class FreelanceProfileRenderer(CSVStreamingRenderer):

    header_fields = OrderedDict([
        ('contributor', 'Contributor'),
        ('payment_date', 'Payment Date'),
        ('pay', 'Pay'),
        ('contributions_count', 'Contributions Count'),
    ])


class LineItemRenderer(CSVStreamingRenderer):

    header_fields = OrderedDict([
        ('payment_date', 'Payment Date'),
        ('payroll_name', 'Payroll Name'),
        ('amount', 'Amount'),
        ('note', 'Note'),
    ])



import json
import sys

from django.core.urlresolvers import resolve, Resolver404
from django.http import HttpResponse, HttpResponseForbidden, HttpResponseNotFound
from django.views.decorators.csrf import csrf_exempt
from django.test.client import RequestFactory

from .models import TargetingOverride


if sys.version_info.major == 3:
    import urllib.parse as urlparse
else:
    from urlparse import urlparse


@csrf_exempt
def targeting(request):
    if not (request.user and request.user.has_perm("ads.change_targetingoverride")):
        return HttpResponseForbidden(
            '{"detail": "You do not have permission to perform this action."}'
        )

    url = request.GET.get("url")
    if url is None:
        return HttpResponseNotFound()

    if url.startswith("http://") or url.startswith("https://"):
        parsed = urlparse(url)
        url = parsed.path

    try:
        view, args, kwargs = resolve(url)
    except Resolver404:
        return HttpResponseNotFound()

    req = RequestFactory().get(url)
    kwargs['request'] = req
    response = view(*args, **kwargs)
    if response.status_code != 200:
        return HttpResponseNotFound()

    targeting = response.context_data.get('targeting', {})

    if request.method == "POST":
        override_data = json.loads(request.body)
        for key, value in targeting.items():
            if value == override_data.get(key):
                del override_data[key]
        override, created = TargetingOverride.objects.get_or_create(url=url)
        override.targeting = override_data
        override.save()
        targeting.update(override_data)
    else:
        try:
            override = TargetingOverride.objects.get(url=url)
            targeting.update(override.targeting)
        except TargetingOverride.DoesNotExist:
            pass
    return HttpResponse(json.dumps(targeting), content_type="application/json")



from elasticsearch_dsl import function, query


def randomize_es(es_queryset):
    """Randomize an elasticsearch queryset."""
    return es_queryset.query(
        query.FunctionScore(
            functions=[function.RandomScore()]
        )
    ).sort("-_score")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields
import json_field.fields
import bulbs.super_features.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0012_auto_20160615_1605'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseSuperFeature',
            fields=[
                ('content_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='content.Content')),
                ('query', json_field.fields.JSONField(default={}, help_text='Enter a valid JSON object', blank=True)),
                ('notes', models.TextField(default=b'', null=True, blank=True)),
                ('internal_name', models.CharField(max_length=255, null=True, blank=True)),
                ('superfeature_type', models.CharField(max_length=255, choices=[(b'GUIDE_TO_HOMEPAGE', b'Guide To Homepage'), (b'GUIDE_TO_ENTRY', b'Guide To Entry')])),
                ('default_child_type', models.CharField(blank=True, max_length=255, null=True, choices=[(b'GUIDE_TO_HOMEPAGE', b'Guide To Homepage'), (b'GUIDE_TO_ENTRY', b'Guide To Entry')])),
                ('data', jsonfield.fields.JSONField(default=dict)),
                ('ordering', models.IntegerField(default=None, null=True, blank=True)),
                ('parent', models.ForeignKey(blank=True, to='super_features.BaseSuperFeature', null=True)),
            ],
            bases=(bulbs.super_features.mixins.SuperFeatureMixin, 'content.content', models.Model),
        ),
        migrations.AlterUniqueTogether(
            name='basesuperfeature',
            unique_together=set([('parent', 'ordering')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import djbetty.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('internal_title', models.CharField(max_length=512)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('headline', models.CharField(max_length=512)),
                ('is_published', models.BooleanField(default=False)),
                ('body', models.TextField(blank=True, null=True)),
                ('image', djbetty.fields.ImageField(blank=True, default=None, null=True)),
                ('clickthrough_url', models.URLField(blank=True, null=True)),
                ('clickthrough_cta', models.CharField(blank=True, max_length=30, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.conf.urls import url

from rest_framework import routers

from bulbs.liveblog.viewsets import LiveBlogEntryViewSet, PublicLiveBlogEntryViewSet

api_v1_router = routers.DefaultRouter()
api_v1_router.register(
    r"liveblog/entry",
    LiveBlogEntryViewSet,
    base_name="liveblog-entry"
)

urlpatterns = [
    url(r"api/v1/liveblog/public/entry/?$",
        PublicLiveBlogEntryViewSet.as_view({'get': 'list'}),
        name="public-liveblog-entry"),
]

urlpatterns += api_v1_router.urls



from rest_framework import serializers

from bulbs.content.serializers import ContentSerializer
from bulbs.super_features.models import BaseSuperFeature
from bulbs.super_features.utils import get_superfeature_model


SUPERFEATURE_MODEL = get_superfeature_model()


class BaseSuperFeatureDataField(serializers.Field):

    def to_internal_value(self, data):
        serializer_class = SUPERFEATURE_MODEL.get_data_serializer(
            self.parent.initial_data.get("superfeature_type")
        )
        return serializer_class().to_internal_value(data)

    def to_representation(self, obj):
        if hasattr(self.parent, 'initial_data'):
            sf_type = self.parent.initial_data.get('superfeature_type', None)
        else:
            sf_type = getattr(self.parent, 'superfeature_type')
        serializer_class = SUPERFEATURE_MODEL.get_data_serializer(sf_type)

        # TODO: figure out a more elegant transition for elasticsearch calls.
        if type(obj).__name__ == 'InnerObjectWrapper':
            obj = obj.__dict__.get('_d_')
        serializer = serializer_class(data=obj)
        serializer.is_valid(raise_exception=True)
        return serializer.data


class BaseSuperFeatureSerializer(ContentSerializer):
    data = BaseSuperFeatureDataField(required=False)
    ordering = serializers.IntegerField(required=False, allow_null=True, default=None)
    parent = serializers.PrimaryKeyRelatedField(
        queryset=BaseSuperFeature.objects.all(), required=False, allow_null=True, default=None
    )
    children_count = serializers.SerializerMethodField()

    class Meta:
        model = BaseSuperFeature

    def to_representation(self, obj):
        self.superfeature_type = getattr(obj, 'superfeature_type')
        return super(BaseSuperFeatureSerializer, self).to_representation(obj)

    def get_children_count(self, obj):
        return SUPERFEATURE_MODEL.objects.filter(parent=obj).count()


class BaseSuperFeaturePartialSerializer(ContentSerializer):

    class Meta:
        model = BaseSuperFeature
        fields = ('id', 'internal_name', 'title')

    def to_representation(self, obj):
        self.superfeature_type = getattr(obj, 'superfeature_type')
        return super(BaseSuperFeaturePartialSerializer, self).to_representation(obj)


class BaseSuperFeatureRelationOrderingSerializer(serializers.Serializer):

    id = serializers.IntegerField(required=True, allow_null=False)
    ordering = serializers.IntegerField(required=True, allow_null=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import os
import re
import sys


name = "django-bulbs"
package = "bulbs"
description = "America's Finest Namespace"
url = "https://github.com/theonion/django-bulbs"
author = "Chris Sinchok"
author_email = "csinchok@theonion.com"
license = "BSD"
requires = [
    "Django>=1.8,<1.9",
    "beautifulsoup4>=4.4.1",
    "celery==3.1.10",
    "contextdecorator==0.10.0",
    "django-enumfield==1.2.1",
    "django-betty-cropper>=0.2.6",
    "django-filter==0.9.2",
    "django-jsonfield==0.9.19",
    # TODO: remove django-json-field from previous JSONField references.
    "django-json-field==0.5.5",
    "django-polymorphic==0.7.1",
    "djangorestframework-csv==1.4.1",
    "pageview_client==1.0.4",
    "djangorestframework==3.1.1",
    "drf-extensions==0.2.8",
    "djes>=0.1.109",
    "drf-nested-routers==0.11.1",
    "firebase-token-generator==1.3.2",
    "python-dateutil==2.1",
    "pytz==2012h",
    "requests>=1.1.0",
    "simplejson==3.3.0",
    "six==1.10.0",
]

dev_requires = [
    "pylint==1.0.0",
    "mock==1.0.1",
    "httmock==1.0.5",
    "model-mommy==1.2.4",
    "psycopg2==2.5.1",
    "pytest==2.7.3",
    "pytest-cov==1.8.1",
    "pytest-django==2.8.0",
    "coveralls==0.4.1",
    "freezegun",
    "vcrpy",
    "ipdb",
    "requests-mock",
    "flake8",
]


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, "__init__.py")).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, "__init__.py"))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, "", 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, "__init__.py"))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    args = {"version": get_version(package)}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s'" % args)
    print("  git push --tags")
    sys.exit()


setup(
    name=name,
    version=get_version(package),
    url=url,
    license=license,
    description=description,
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=requires,
    extras_require={
        'dev': dev_requires,
    },
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('liveblog', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='liveblogresponse',
            name='internal_name',
            field=models.CharField(max_length=255, blank=True, null=True),
        ),
    ]



from django.conf import settings

from rest_framework.exceptions import APIException


SODAHEAD_DATE_FORMAT = '%m/%d/%y %I:%M %p'

SODAHEAD_POLL_ENDPOINT = '{}/api/polls/{{}}/'.format(settings.SODAHEAD_BASE_URL)
SODAHEAD_POLLS_ENDPOINT = '{}/api/polls/'.format(settings.SODAHEAD_BASE_URL)
SODAHEAD_DELETE_POLL_ENDPOINT = '{}/api/polls/{{}}/?access_token={{}}'.format(
    settings.SODAHEAD_BASE_URL
)

BLANK_ANSWER = 'Intentionally blank'
DEFAULT_ANSWER_1 = 'default answer 1'
DEFAULT_ANSWER_2 = 'default answer 2'


class SodaheadResponseError(APIException):
    status_code = 503
    default_detail = "Third-party poll provider temporarily unavailable."

    def __init__(self, detail):
        self.detail = detail


class SodaheadResponseFailure(APIException):
    status_code = 400
    default_detail = "Error from third-party poll provider"

    def __init__(self, detail):
        self.detail = detail



from rest_framework.metadata import SimpleMetadata
from rest_framework.utils.field_mapping import ClassLookupDict

from djbetty.serializers import ImageFieldSerializer

from bulbs.content.serializers import AuthorField
from bulbs.utils.fields import RichTextField
from bulbs.utils.metadata import BaseSimpleMetadata
from bulbs.utils.data_serializers import CopySerializer, EntrySerializer
from bulbs.super_features.serializers import BaseSuperFeatureDataField
from bulbs.super_features.utils import get_superfeature_serializer

SUPERFEATURE_SERIALIZER = get_superfeature_serializer()


class BaseSuperFeatureMetadata(BaseSimpleMetadata):
    custom_serializer = SUPERFEATURE_SERIALIZER
    custom_data_field = BaseSuperFeatureDataField

    @property
    def label_lookup(self):
        mapping = SimpleMetadata.label_lookup.mapping
        mapping.update({
            AuthorField: "string",
            CopySerializer: "array",
            EntrySerializer: "array",
            ImageFieldSerializer: "image",
            RichTextField: "richtext"
        })
        return ClassLookupDict(mapping)

    def get_custom_field_name(self, view):
        obj = view.get_object()
        serializer = obj.get_data_serializer(obj.superfeature_type)
        return self.get_custom_metadata(serializer, view)



from django.core.management.base import BaseCommand
from django.shortcuts import get_object_or_404

from bulbs.content.models import FeatureType


class Command(BaseCommand):

    help = "Set a provided FeatureType as available for Facebook Instant Articles."

    def add_arguments(self, parser):
        parser.add_argument(
            "--slug",
            dest="featuretype_slug",
            help="The slug for the desired FeatureType.",
            required=True,
            type=str
        )

    def handle(self, *args, **kwargs):
        featuretype_slug = kwargs.get("featuretype_slug")
        featuretype = get_object_or_404(FeatureType, slug=featuretype_slug)
        if not featuretype.instant_article:
            featuretype.instant_article = True
            featuretype.save()
            # celery is not configured during commands.
            for content in featuretype.content_set.all():
                content.index()



from django.apps import apps
from django.conf import settings
from django.http import Http404, HttpResponseRedirect
from django.utils import timezone
from django.views.generic.base import View
from django.views.decorators.cache import cache_control

################################################################################
# FIXME: Tuesday, August 12 (2016) Collin Miller:
# we are temporarily overriding the
# video redirect url `/v/3838` on theonion.com to work around some of our content
# items that reference the same videohub id.
# We are overriding the logic to prefer redirecting to the url for some
# FeatureTypes over others. Once editorial has pruned down the duplicates we
# will add a unique constraint on the videohub_ref_id and we can go back to the
# default redirect logic.
#
# The method temporary_workaround_to_choose_from_duplicate_videos will not be
# needed once this unique constraint is setup.
################################################################################


class VideoRedirectView(View):
    def get(self, request, videohub_ref_id):
        videos = self.get_videos_for_redirect(videohub_ref_id)
        video = self.temporary_workaround_to_choose_from_duplicate_videos(videos)

        if video is None:
            raise Http404("No video found with the provided videohub_ref id.")

        return HttpResponseRedirect(video.get_absolute_url())

    def temporary_workaround_to_choose_from_duplicate_videos(self, videos):
        if videos:
            return videos[0]

    def get_videos_for_redirect(self, videohub_ref_id):
        video_model_name = getattr(settings, "VIDEO_MODEL", "")
        video_model = apps.get_model(video_model_name)
        videos = video_model.objects.filter(
            videohub_ref__id=int(videohub_ref_id),
            published__lte=timezone.now(),
        ).order_by(
            '-published'
        ).all()

        return videos

video_redirect = cache_control(max_age=3600)(VideoRedirectView.as_view())



import re

from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible
from django.utils.encoding import force_text


@deconstructible
class ColorValidator(object):
    message = ('Enter a valid hexadecimal color.')
    code = 'invalid'
    color_regex = r'^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$'

    def __init__(self, message=None, code=None):
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code

    def __call__(self, value):
        value = force_text(value)
        if not re.compile(self.color_regex, re.IGNORECASE).match(value):
            raise ValidationError(self.message, code=self.code)

    def __eq__(self, other):
        return (
            isinstance(other, ColorValidator) and
            (self.message == other.message),
            (self.code == other.code)
        )
